@@ -1,4 +1,3 @@
-import json
 import requests
 
 from akinoncli.core.exc import AkinonCLIError
@@ -19,6 +18,7 @@ class AkinonCloudClient:
         "app_detail": "applications/{app_id}/",
         "app_build": "applications/{app_id}/build/",
         "app_versions": "applications/{app_id}/versions/",
+        "app_version_logs": "applications/{app_id}/versions/{v_id}/logs",
         "app_version_detail": "applications/{app_id}/versions/{v_id}/",
         "application_types": "applications/application_types/",
         "application_types_admin": "admin/application_types/",
@@ -32,7 +32,11 @@ class AkinonCloudClient:
         "project_app_detail": "projects/{p_id}/project_apps/{pa_id}/",
         "project_app_custom_env": "projects/{p_id}/project_apps/{pa_id}/custom_env/",
         "project_app_deploy": "projects/{p_id}/project_apps/{pa_id}/deploy/",
+        "project_app_deploy_multiple": "applications/{app_id}/deploy_multiple_project_apps/",
+        "project_app_deploy_stable_version": "projects/{project_id}/project_apps/{project_app_id}/deploy_stable/",
+        "project_app_redeploy": "projects/{project_id}/project_apps/{project_app_id}/redeploy/",
         "project_app_deployments": "projects/{p_id}/project_apps/{pa_id}/deployments/",
+        "project_app_deployment_log": "projects/{p_id}/project_apps/{pa_id}/deployments/{deployment_id}/logs",
         "project_apps_logs": "projects/{project_id}/project_apps/{project_app_id}/logs/",
         "project_apps_export_logs": "projects/{project_id}/project_apps/{project_app_id}/logs/export",
         "public_keys": "users/public_keys/",
@@ -124,6 +128,9 @@ class AkinonCloudClient:
     def get_app_versions(self, app_id):
         return self._send_request("GET", "app_versions", params={"app_id": app_id})
 
+    def get_app_version_logs(self, app_id, version_id):
+        return self._send_request("GET", "app_version_logs", params={"app_id": app_id, "v_id": version_id})
+
     def get_app_users(self, app_id):
         return self._send_request("GET", "app_users", params={"app_id": app_id})
 
@@ -197,18 +204,70 @@ class AkinonCloudClient:
         return self._send_request("POST", "project_apps", params={"p_id": p_id},
                                   data=data)
 
-    def deploy_project_app(self, p_id, pa_id, data):
-        response = self._send_request("POST", "project_app_deploy", params={"p_id": p_id,
-                                                                            "pa_id": pa_id},
-                                      data=data)
+    def deploy_project_app(self, p_id: int, pa_id: int, data: dict) -> AkinonResponse:
+        response = self._send_request(
+            "POST",
+            "project_app_deploy",
+            params={
+                "p_id": p_id,
+                "pa_id": pa_id,
+            },
+            data=data
+        )
         if response.status_code == 404:
-            raise AkinonCLIError("Project App could not found.")
+            raise AkinonCLIError("Project App could not be found.")
+        return response
+
+    def deploy_multiple_project_apps(self, app_id: int, data: dict) -> AkinonResponse:
+        response = self._send_request(
+            "POST",
+            "project_app_deploy_multiple",
+            params={
+                "app_id": app_id,
+            },
+            data=data
+        )
+        if response.status_code == 404:
+            raise AkinonCLIError("App could not be found.")
+        return response
+
+    def deploy_project_app_with_stable_version(self, project_id: int, project_app_id: int) -> AkinonResponse:
+        response = self._send_request(
+            "POST",
+            "project_app_deploy_stable_version",
+            params={
+                "project_id": project_id,
+                "project_app_id": project_app_id,
+            },
+            response_format=None,
+        )
+        if response.status_code == 404:
+            raise AkinonCLIError("Project App could not be found.")
+        return response
+
+    def deploy_project_app_with_autolatest(self, project_id: int, project_app_id: int) -> AkinonResponse:
+        response = self._send_request(
+            "POST",
+            "project_app_redeploy",
+            params={
+                "project_id": project_id,
+                "project_app_id": project_app_id,
+            },
+            response_format=None,
+        )
+        if response.status_code == 404:
+            raise AkinonCLIError("Project App could not be found.")
         return response
 
     def get_project_app_deployments(self, p_id, pa_id):
         return self._send_request("GET", "project_app_deployments", params={"p_id": p_id,
                                                                             "pa_id": pa_id})
 
+    def get_project_app_deployment_logs(self, p_id, pa_id, deployment_id):
+        return self._send_request("GET", "project_app_deployment_log", params={"p_id": p_id,
+                                                                               "pa_id": pa_id,
+                                                                               "deployment_id": deployment_id})
+
     def get_public_keys(self, qs: dict):
         return self._send_request("GET", "public_keys", qs=qs)
 
