@@ -67,9 +67,9 @@ class Projects(Controller):
         }
         response = self.app.client.create_project(data)
         if response.is_succeed:
-            self.app.render({}, is_text=True, custom_text="Project has been created.")
+            self.app.render({}, renderer_type='text', custom_text="Project has been created.")
         else:
-            self.app.render({}, is_text=True, custom_text=response.data['slug'][0])
+            self.app.render({}, renderer_type='text', custom_text=response.data['slug'][0])
 
     @ex(
         help='Project Update Command',
@@ -91,9 +91,9 @@ class Projects(Controller):
         }
         response = self.app.client.update_project(p_id, data)
         if response.is_succeed:
-            self.app.render({}, is_text=True, custom_text="Project has been updated.")
+            self.app.render({}, renderer_type='text', custom_text="Project has been updated.")
         else:
-            self.app.render({}, is_text=True, custom_text=response.data)
+            self.app.render({}, renderer_type='text', custom_text=response.data)
 
 
 class ProjectApps(Controller):
@@ -157,8 +157,9 @@ class ProjectApps(Controller):
         project_app = self.app.client.get_project_app(p_id, pa_id)
         if response.is_succeed:
             if self.app.pargs.deploy:
-                current_deployment_tag = {'tag': project_app.data['current_deployment']['version']}
-                self.app.client.deploy_project_app(p_id, pa_id, data=current_deployment_tag)
+                self.app.client.deploy_project_app_with_autolatest(
+                    project_id=p_id, project_app_id=pa_id
+                )
 
             row = response.data
             row['env'] = ''
@@ -174,7 +175,7 @@ class ProjectApps(Controller):
                                      'created_date': 'Created Date', 'env': 'ENV Variables'},
                             is_succeed=response.is_succeed)
         else:
-            self.app.render({}, is_text=True, custom_text=response.data)
+            self.app.render({}, renderer_type='text', custom_text=response.data)
 
     @ex(
         help='Project App Remove Environment Value Command',
@@ -205,8 +206,9 @@ class ProjectApps(Controller):
 
         if response.is_succeed:
             if self.app.pargs.deploy:
-                current_deployment_tag = {'tag': project_app.data['current_deployment']['version']}
-                self.app.client.deploy_project_app(p_id, pa_id, data=current_deployment_tag)
+                self.app.client.deploy_project_app_with_autolatest(
+                    project_id=p_id, project_app_id=pa_id
+                )
 
             row = response.data
             row['env'] = ''
@@ -223,7 +225,7 @@ class ProjectApps(Controller):
                                      'created_date': 'Created Date', 'env': 'ENV Variables'},
                             is_succeed=response.is_succeed)
         else:
-            self.app.render({}, is_text=True, custom_text=response.data)
+            self.app.render({}, renderer_type='text', custom_text=response.data)
 
     @ex(
         help='Project App Add Command',
@@ -246,9 +248,9 @@ class ProjectApps(Controller):
         }
         response = self.app.client.create_project_app(p_id, data)
         if response.is_succeed:
-            self.app.render({}, is_text=True, custom_text="App has been added to the project.")
+            self.app.render({}, renderer_type='text', custom_text="App has been added to the project.")
         else:
-            self.app.render({}, is_text=True, custom_text=response.data)
+            self.app.render({}, renderer_type='text', custom_text=response.data)
 
     @ex(
         help='Project App Deploy Command',
@@ -276,12 +278,49 @@ class ProjectApps(Controller):
         response = self.app.client.deploy_project_app(p_id, pa_id, data=data)
 
         if response.is_succeed:
-            self.app.render({}, is_text=True, custom_text="ProjectApp deployment has been started.")
+            self.app.render({}, renderer_type='text', custom_text="ProjectApp deployment has been started.")
         else:
-            self.app.render({}, is_text=True, custom_text=response.data['non_field_errors'])
+            self.app.render({}, renderer_type='text', custom_text=response.data['non_field_errors'])
 
     @ex(
-        help='Project App Deployment List Command',
+        help='Multiple Project App Deploy Command',
+        arguments=[
+            (['app_id'], {
+                'help': 'Application ID',
+                'action': 'store',
+            }),
+            (['tag'], {
+                'help': 'App Version',
+                'action': 'store',
+            }),
+            (['--project-apps'], {
+                'help': 'Project App IDs',
+                'action': 'store',
+                'nargs': '+',
+                'default': [],
+            }),
+            (['--deploy-all'], {
+                'help': "Deploys all project apps with given app_id and tag.",
+                'action': 'store_true',
+                'dest': 'deploy_all',
+                'default': False,
+            }),
+        ]
+    )
+    def deploy_multiple(self):
+        data = {
+            'tag': self.app.pargs.tag,
+            'project_apps': self.app.pargs.project_apps or [],
+            'deploy_all': self.app.pargs.deploy_all,
+        }
+        response = self.app.client.deploy_multiple_project_apps(self.app.pargs.app_id, data)
+        if response.is_succeed:
+            self.app.render({}, renderer_type='text', custom_text="ProjectApp deployments have been started.")
+        else:
+            self.app.render({}, renderer_type='text', custom_text=response.data["non_field_errors"])
+
+    @ex(
+        help='Deploy Project App With Stable Version',
         arguments=[
             (['project_id'], {
                 'help': 'Project ID',
@@ -293,6 +332,58 @@ class ProjectApps(Controller):
             }),
         ],
     )
+    def deploy_stable(self):
+        project_id = self.app.pargs.project_id
+        project_app_id = self.app.pargs.project_app_id
+
+        response = self.app.client.deploy_project_app_with_stable_version(
+            project_id=project_id, project_app_id=project_app_id
+        )
+    
+        if response.is_succeed:
+            self.app.render({}, renderer_type='text', custom_text="ProjectApp deployment has been started with the stable version.")
+        else:
+            self.app.render({}, renderer_type='text', custom_text=response.data['non_field_errors'])
+
+    @ex(
+        help='Deploy Project App With Last Completed Deployment',
+        arguments=[
+            (['project_id'], {
+                'help': 'Project ID',
+                'action': 'store',
+            }),
+            (['project_app_id'], {
+                'help': 'Project App ID',
+                'action': 'store',
+            }),
+        ],
+    )
+    def redeploy(self):
+        project_id = self.app.pargs.project_id
+        project_app_id = self.app.pargs.project_app_id
+
+        response = self.app.client.deploy_project_app_with_autolatest(
+            project_id=project_id, project_app_id=project_app_id
+        )
+    
+        if response.is_succeed:
+            self.app.render({}, renderer_type='text', custom_text="ProjectApp deployment has been started with the last completed deployment.")
+        else:
+            self.app.render({}, renderer_type='text', custom_text=response.data['non_field_errors'])
+
+    @ex(
+        help='Project App Deployment List Command',
+        arguments=[
+            (['project_id'], {
+                'help': 'Project ID',
+                'action': 'store',
+            }),
+            (['project_app_id'], {
+                'help': 'Project App ID',
+                'action': 'store',
+            })
+        ],
+    )
     def deployments(self):
         p_id = self.app.pargs.project_id
         pa_id = self.app.pargs.project_app_id
@@ -302,6 +393,29 @@ class ProjectApps(Controller):
                                  'created_date': 'Created Date'},
                         is_succeed=response.is_succeed)
 
+    @ex(
+        help='Project App Deployment Log List Command',
+        arguments=[
+            (['project_id'], {
+                'help': 'Project ID',
+                'action': 'store',
+            }),
+            (['project_app_id'], {
+                'help': 'Project App ID',
+                'action': 'store',
+            }),
+            (['deployment_id'], {'help': 'Deployment ID', 'action': 'store'})
+        ],
+    )
+    def deployment_logs(self):
+        p_id = self.app.pargs.project_id
+        pa_id = self.app.pargs.project_app_id
+        deployment_id = self.app.pargs.deployment_id
+        response = self.app.client.get_project_app_deployment_logs(p_id, pa_id, deployment_id=deployment_id)
+        self.app.render(
+            data=response.data, rows=response.data, renderer_type="pipeline_log", is_succeed=response.is_succeed
+        )
+
     @ex(
         help='Attach Certificate to Project App',
         arguments=[
@@ -328,11 +442,11 @@ class ProjectApps(Controller):
         response = self.app.client.update_project_app(p_id, pa_id, data=data)
 
         if response.is_succeed:
-            self.app.render({}, is_text=True,
+            self.app.render({}, renderer_type='text',
                             custom_text="Certificate has been attached"
                                         " to project app.")
         else:
-            self.app.render({}, is_text=True,
+            self.app.render({}, renderer_type='text',
                             custom_text=response.data['non_field_errors'])
 
     @ex(
@@ -357,11 +471,11 @@ class ProjectApps(Controller):
         response = self.app.client.update_project_app(p_id, pa_id, data=data)
 
         if response.is_succeed:
-            self.app.render({}, is_text=True,
+            self.app.render({}, renderer_type='text',
                             custom_text="Certificate has been detached"
                                         " from project app.")
         else:
-            self.app.render({}, is_text=True,
+            self.app.render({}, renderer_type='text',
                             custom_text=response.data['non_field_errors'])
 
     @ex(
@@ -396,10 +510,10 @@ class ProjectApps(Controller):
             if 'message' not in row:
                 row['message'] = ""
         self.app.render(
+            renderer_type="realtime_log",
             data=response.data,
             rows=rows,
-            is_succeed=response.is_succeed,
-            is_log=True,
+            is_succeed=response.is_succeed
         )
 
     @ex(
@@ -446,7 +560,7 @@ class ProjectApps(Controller):
             else:
                 raise
 
-        self.app.render({}, custom_text=response.data['message'], is_text=True)
+        self.app.render({}, custom_text=response.data['message'], renderer_type='text')
 
 
 class Addons(Controller):
